#!/usr/bin/python
"""
TTT TTS Controller Setup Script
Christopher Woodall <cwoodall at bu.edu>
Boston University EDF <http://edf.bu.edu>
January 15, 2012

Version 1

Setup TTS_control fabric on the TTT board over IPBus.

@requires pychips
"""
VERSION=1
print "TTT TTS Controller Setup Script v%d" % VERSION
print "Boston University EDF"
print ""
from PyChipsUser import *
import sys
import os
from optparse import OptionParser

ctrl_reg = 0x0
action_reg = 0x0
rule_count = [0,0,0,0]
def store_rules(option, opt_str, value, parser):
    if (value[0] < 1 or value[0] > 4):
        print "Rule number must be 1-4"
        sys.exit(1)
    else:
        rule_count[value[0]-1] = value[1]
        
# Setup OptionParser
parser = OptionParser()
parser.add_option("-o", dest="orbit_length", help="orbit length",
                  default=3563, metavar="<len>", type=int)
parser.add_option("-d", dest="trig_delay", help="trigger delay",
                  default=100, metavar="<dly>", type=int)
parser.add_option("-n", dest="orbit_count", help="number of orbits",
                  default=10, metavar="<n>", type=int)
parser.add_option("-s", dest="trig_spacing", help="trigger spacing",
                  default=25, metavar="<spc>", type=int)
parser.add_option("-t", dest="trig_per_orbit", help="triggers per orbit",
                  default=4, metavar="<n>", type=int)
parser.add_option("-r", dest="repeat_period",
                  help="repeat period (orbits, 0 for no repeat).",
                  default=0, metavar="<per>", type=int)
parser.add_option("-b", dest="backpressure", action="store_true",
                  help="enable backpressure", default=False)
parser.add_option("-p", dest="random_period",
                  help="Random threshold 155 = 100kHz.",
                  default=0, metavar="<thr>", type=int)
parser.add_option("-L", dest="latency",
                  help="Latency 0-255 for TTS backpressure.",
                  default=0, metavar="<lat>", type=int)
parser.add_option("-m", dest="tts_mask", help="TTS Simple Mask",
                  default=0, metavar="<mask>", type=int)
parser.add_option("-w", dest="ttc_bcn", help="TTC trigger output BcN",
                  default=1000, metavar="<bcn>", type=int)
parser.add_option("-g", action="store_true", dest="allow_gap",
                  help="Allow L1A in orbit gap.", default=False)
parser.add_option("-z", dest="front_back_porch",
                  help="Front and Back porch of gap (orbit gap is BC0-b to BC0+f).",
                  default=(250, 500), metavar="<f> <b>", nargs=2, type=int)
parser.add_option("-u", dest="rule_no_counts", help="Rule number and rule count",
                  nargs=2, action="callback", metavar="<num> <count>",
                  callback=store_rules, type=int)

parser.add_option("-i", "--ip-address", type=str, metavar="<ip-address>:<port>",
                  dest="ip_address", default="192.168.1.6:50001",
                  help="Set IP Address and port for IPBUs TTT device")

(options, args) = parser.parse_args()

ip_address_raw = options.ip_address.split(":")
ip_address = ""
port = 50001

if (len(ip_address_raw) == 1): 
    ip_address = ip_address_raw[0]    
elif (len(ip_address_raw) == 2):
    ip_address = ip_address_raw[0]
    port = int(ip_address_raw[1])
else:
    print "InputError: IP Address %s, not formatted correctly." % options.ip_address
    print "            Must be formatted as ip-address[:port], where the things in brackets are options"
    sys.exit(1)

# Connect to TTT board over IPBus

# @todo: add way to specify address table in arguments
addrTable = AddressTable(str(os.environ["TTT_ADDRESS_TABLE"]))
# @todo: add way to specify IP Address and Port #
print("Connecting to %s:%s" % (ip_address, port))
TTTBoard = ChipsBusUdp(addrTable, ip_address, port)
try:
    print("TTT Firmware Version: %s" % hex(TTTBoard.read("FirmwareVersionID")))
    print("")

except ChipsException:
    print("IPBus Error: Cannot open connection to TTTBoard at %s:%s" % (ip_address, port))
    print("Exiting")
    sys.exit(1)
    
print("Orbit Length:       -o %d BX" % options.orbit_length)
print("Trigger Delay:      -d %d BX" % options.trig_delay)
print("Orbit Count:        -n %d orbits" % options.orbit_count)
print("Trigger Spacing:    -s %d BX" % options.trig_spacing)
print("Triggers per orbit: -t %d triggers" % options.trig_per_orbit)
print("Repeat period:      -r %d orbits" % options.repeat_period)
print("Random threshold:   -p %d / 65535" % options.random_period)
print("TTS latency         -l %d BXn (%g sec)" % (options.latency,
      options.latency * 25e-9))
print("TTS sample mask     -m %d" % options.tts_mask)
print("TTC cmd BCN         -w %d" % options.ttc_bcn)
print("Allow L1A in gap    -g %d" % options.allow_gap)
print("Gap window          -z %d %d" % (options.front_back_porch[0],
      options.front_back_porch[1]))

print("")
# Disable any ongoing repeats 
TTTBoard.write("TTSControlReg", 0x00)

TTTBoard.write("TTSOrbitLength", options.orbit_length)
TTTBoard.write("TTSTrigStart", options.trig_delay)
TTTBoard.write("TTSOrbitMax", options.orbit_count)
TTTBoard.write("TTSL1ASpacing", options.trig_spacing)
TTTBoard.write("TTSL1APerOrbit", options.trig_per_orbit)
TTTBoard.write("TTSRepeatPeriod", options.repeat_period)
TTTBoard.write("TTSRateSet", options.random_period)
TTTBoard.write("TTSSetBPDelay", options.latency)
TTTBoard.write("TTSBPSampleMask", options.tts_mask)
TTTBoard.write("TTSTTCBxNum", options.ttc_bcn)

# Currently can't alter front and back porch
#TTTBoard.write("TTSFrontPorch", options.front_back_porch[0])
#TTTBoard.write("TTSBackPorch", options.front_back_porch[1])

# Setup trigger rules
TTTBoard.blockWrite("TTSRule1", rule_count)
v = 0
for i, item in enumerate(rule_count):
    if item:
        print("No more than %d triggers per %d BX" % (i+1, item))
        v = v | (1<<i)
    else:
        print "Not enabled"

print("Rule enable mask: 0x%x\n" % v)
TTTBoard.write("TTSClkSel", v)

if( options.repeat_period > 0):
    ctrl_reg |= 2
    
if options.trig_per_orbit > 1:
    ctrl_reg |= 1

if options.backpressure:
    ctrl_reg |= 4

if options.random_period:
    ctrl_reg |= 8

if options.allow_gap:
    ctrl_reg |= 0x10

TTTBoard.write("TTSControlReg", ctrl_reg)
TTTBoard.write("TTSActionReg", 0x80) # RESET
TTTBoard.write("TTSActionReg", 0x1) # Trigger desired sequence

print "My job here is done. Enjoy your triggers!"
